LIBRARY ieee;
USE ieee.std_logic_1164.all;
USE ieee.std_logic_unsigned.all;

PACKAGE JH_384_paket IS	

	TYPE message_type IS ARRAY(63 DOWNTO 0) OF STD_LOGIC_VECTOR(7 DOWNTO 0); 
	TYPE hash_type IS ARRAY(127 DOWNTO 0) OF STD_LOGIC_VECTOR(7 DOWNTO 0); 
	TYPE round_type IS ARRAY(255 DOWNTO 0) OF STD_LOGIC_VECTOR(3 DOWNTO 0); 
	TYPE round_zero_type IS ARRAY(63 DOWNTO 0) OF STD_LOGIC_VECTOR(3 DOWNTO 0); 
	TYPE s_box_type IS ARRAY(0 TO 15) OF STD_LOGIC_VECTOR(3 DOWNTO 0); 
	TYPE final_automat IS (init, round_init, processing_init,idle, round, processing_round, processing, finalization); 	

	PROCEDURE s_box0_new(VARIABLE element: INOUT STD_LOGIC_VECTOR(3 DOWNTO 0));
	PROCEDURE s_box1_new(VARIABLE element: INOUT STD_LOGIC_VECTOR(3 DOWNTO 0));
	PROCEDURE update_roundconstant(VARIABLE roundconstant: INOUT round_zero_type);
	PROCEDURE r8(VARIABLE roundconstant: IN round_zero_type; VARIABLE Q: INOUT round_type);
	PROCEDURE e8_degrouping(VARIABLE Q: IN round_type; VARIABLE A: OUT hash_type);
	PROCEDURE e8_grouping (VARIABLE A: IN hash_type; VARIABLE Q: OUT round_type);	
	PROCEDURE f8(VARIABLE bufer: IN message_type; VARIABLE H: INOUT hash_type;VARIABLE Q: OUT round_type);
	PROCEDURE f8_reverse(VARIABLE Q: IN round_type; VARIABLE H: INOUT hash_type);
	PROCEDURE hash_initial (VARIABLE H: INOUT hash_type;VARIABLE Q: OUT round_type);
	PROCEDURE hash_calculate (SIGNAL message_block: IN STD_LOGIC_VECTOR(511 DOWNTO 0); VARIABLE H: INOUT hash_type;VARIABLE Q: INOUT round_type);
	PROCEDURE hash_calculate_reverse (SIGNAL message_block: IN STD_LOGIC_VECTOR(511 DOWNTO 0); VARIABLE Q: INOUT round_type;VARIABLE H: INOUT hash_type);
	PROCEDURE hash_final_func (VARIABLE H: INOUT hash_type; VARIABLE hash_final: OUT STD_LOGIC_VECTOR (383 DOWNTO 0));
	PROCEDURE roundconstant_expanded_func(VARIABLE roundconstant: INOUT round_zero_type);

	CONSTANT roundconstant_zero: STD_LOGIC_VECTOR (255 DOWNTO 0):= X"6a09e667f3bcc908b2fb1366ea957d3e3adec17512775099da2f590b0667322a";
	CONSTANT hashbitlen: STD_LOGIC_VECTOR (15 DOWNTO 0):= X"0180";
	CONSTANT s_box0: s_box_type:=
	(
	X"9", X"0", X"4", X"b", X"d", X"c", X"3", X"f", X"1", X"a", X"2", X"6", X"7", X"5", X"8", X"e" 
	);
	CONSTANT s_box1: s_box_type:=
	(
	X"3", X"c", X"6", X"d", X"5", X"7", X"1", X"9", X"f", X"2", X"0", X"4", X"b", X"a", X"e", X"8"
	); 
END JH_384_paket;

PACKAGE BODY JH_384_paket IS 

	PROCEDURE s_box0_new(VARIABLE element: INOUT STD_LOGIC_VECTOR(3 DOWNTO 0)) IS
		VARIABLE index: INTEGER RANGE 0 TO 15;
	BEGIN  
		index:=conv_integer(element);
		element:= s_box0(index);  
	END s_box0_new;
	
	PROCEDURE s_box1_new(VARIABLE element: INOUT STD_LOGIC_VECTOR(3 DOWNTO 0)) IS
		VARIABLE index: INTEGER RANGE 0 TO 15;
	BEGIN  
		index:=conv_integer(element);
		element:= s_box1(index);  
	END s_box1_new;

	PROCEDURE update_roundconstant(VARIABLE roundconstant: INOUT round_zero_type) IS
 		VARIABLE v,w,f,g,h: round_zero_type;		
		VARIABLE index: INTEGER RANGE 0 TO 15;
	BEGIN						
		FOR i IN 0 TO 63 LOOP		
			s_box0_new(roundconstant(i));
		END LOOP;	
		FOR i IN 0 TO 31 LOOP								 
			w(2*i+1)(0):=roundconstant(2*i+1)(0) XOR roundconstant(2*i)(3);
			w(2*i+1)(1):=roundconstant(2*i+1)(1) XOR roundconstant(2*i)(0)  XOR roundconstant(2*i)(3);
			w(2*i+1)(2):=roundconstant(2*i+1)(2) XOR roundconstant(2*i)(1);
			w(2*i+1)(3):=roundconstant(2*i+1)(3) XOR roundconstant(2*i)(2);
			w(2*i)(0):=roundconstant(2*i)(0) XOR w(2*i+1)(3);
			w(2*i)(1):=roundconstant(2*i)(1) XOR w(2*i+1)(0) XOR w(2*i+1)(3);
			w(2*i)(2):=roundconstant(2*i)(2) XOR w(2*i+1)(1);
			w(2*i)(3):=roundconstant(2*i)(3) XOR w(2*i+1)(2);			  
		END LOOP; 
		FOR i IN 0 TO 15 LOOP
			roundconstant(2*i):=w(4*i);
			roundconstant(2*i+1):=w(4*i+3);
			roundconstant(2*i+32):=w(4*i+2);
			roundconstant(2*i+33):=w(4*i+1);
		END LOOP; 
	END update_roundconstant;
	
	PROCEDURE r8(VARIABLE roundconstant: IN round_zero_type; VARIABLE Q: INOUT round_type) IS    
		VARIABLE v,w: round_type;
		VARIABLE index: INTEGER RANGE 0 TO 15;		
		VARIABLE roundconstant_expanded: STD_LOGIC_VECTOR(255 DOWNTO 0); 	 
	BEGIN  			
		FOR i IN 0 TO 63 LOOP 
			FOR k IN 0 TO 3 LOOP					
				roundconstant_expanded(4*i+k) := roundconstant(i)(3-k); 
				IF (roundconstant_expanded(4*i+k)='0') THEN 
					s_box0_new(Q(4*i+k)(3 DOWNTO 0));
				ELSE
					s_box1_new(Q(4*i+k)(3 DOWNTO 0));
		   	END IF;
			END LOOP;	
		END LOOP;		
		FOR i IN 0 TO 127 LOOP 	 
			w(2*i+1)(0):=Q(2*i+1)(0) XOR Q(2*i)(3);
			w(2*i+1)(1):=Q(2*i+1)(1) XOR Q(2*i)(0) XOR Q(2*i)(3);
			w(2*i+1)(2):=Q(2*i+1)(2) XOR Q(2*i)(1);
			w(2*i+1)(3):=Q(2*i+1)(3) XOR Q(2*i)(2);
			w(2*i)(0):=Q(2*i)(0) XOR w(2*i+1)(3);
			w(2*i)(1):=Q(2*i)(1) XOR w(2*i+1)(0) XOR w(2*i+1)(3);
			w(2*i)(2):=Q(2*i)(2) XOR w(2*i+1)(1);
			w(2*i)(3):=Q(2*i)(3) XOR w(2*i+1)(2);				
		END LOOP;
		FOR i IN 0 TO 63 LOOP									
			Q(2*i):=w(4*i);
			Q(2*i+1):=w(4*i+3);
			Q(2*i+128):=w(4*i+2);
			Q(2*i+129):=w(4*i+1);
		END LOOP; 
	END r8;

	PROCEDURE e8_degrouping(VARIABLE Q: IN round_type;VARIABLE A: OUT hash_type) IS     
	   VARIABLE q0,q1,q2,q3,q0_new,q1_new,q2_new,q3_new: STD_LOGIC;
		VARIABLE brojac: INTEGER RANGE 0 TO 15;
		VARIABLE brojac1: INTEGER RANGE 0 TO 31;
		VARIABLE tmp:STD_LOGIC_VECTOR(3 DOWNTO 0);		
	BEGIN 
		FOR i IN 0 TO 15 LOOP
			FOR j IN 0 TO 7 LOOP 
			   A(i)(7-j):=Q(2*i*8+2*j)(3);
				A(i+32)(7-j):=Q(2*i*8+2*j)(2);
				A(i+64)(7-j):=Q(2*i*8+2*j)(1);
				A(i+96)(7-j):=Q(2*i*8+2*j)(0);
			   A(i+16)(7-j):=Q(2*i*8+2*j+1)(3);
				A(i+32+16)(7-j):=Q(2*i*8+2*j+1)(2);
				A(i+64+16)(7-j):=Q(2*i*8+2*j+1)(1);
				A(i+96+16)(7-j):=Q(2*i*8+2*j+1)(0);
				END LOOP;		 
		END LOOP;   			
	END e8_degrouping;
	
	PROCEDURE e8_grouping (VARIABLE A: IN hash_type; VARIABLE Q: OUT round_type) IS		
		VARIABLE q0,q1,q2,q3,q0_new,q1_new,q2_new,q3_new: STD_LOGIC;
		VARIABLE brojac: INTEGER RANGE 0 TO 31;
		VARIABLE brojac1: INTEGER RANGE 0 TO 31;
		VARIABLE tem: round_type;		
	BEGIN
		FOR i IN 0 TO 15 LOOP
			FOR j IN 0 TO 7 LOOP 				
				q0 := A(i)(7-j); 
				q1 := A(i+32)(7-j); 
				q2 := A(i+64)(7-j);
				q3:=  A(i+96)(7-j); 
				q0_new := A(i+16)(7-j); 
				q1_new := A(i+32+16)(7-j); 
				q2_new := A(i+64+16)(7-j);
				q3_new:=  A(i+96+16)(7-j);
				Q(2*i*8+2*j)(3 DOWNTO 0) := q0 & q1 & q2 & q3; 
				Q(2*i*8+2*j+1)(3 DOWNTO 0) := q0_new & q1_new & q2_new & q3_new;
				END LOOP;
			END LOOP;        			 
	END e8_grouping;	
	
	PROCEDURE roundconstant_expanded_func(VARIABLE roundconstant: INOUT round_zero_type) IS	
		 VARIABLE roundconstant_zero_var: STD_LOGIC_VECTOR (255 DOWNTO 0);
	BEGIN
		roundconstant_zero_var:= roundconstant_zero;
		FOR i IN 0 TO 63 LOOP 
			roundconstant(i)(3 DOWNTO 0) := roundconstant_zero_var(255 DOWNTO 252); 
			roundconstant_zero_var:= SHL(roundconstant_zero_var, "0100");
		END LOOP;		
	END roundconstant_expanded_func;
	
	PROCEDURE f8(VARIABLE bufer: IN message_type; VARIABLE H: INOUT hash_type;VARIABLE Q: OUT round_type) IS	
	BEGIN
		FOR i IN 63 DOWNTO 0 LOOP
			FOR j IN 7 DOWNTO 0 LOOP
				H(i)(j):= H(i)(j) XOR bufer(i)(j); 
			END LOOP;
		END LOOP;
		e8_grouping(H,Q);			
	END f8;
	
	PROCEDURE f8_reverse(VARIABLE Q: IN round_type; VARIABLE H: INOUT hash_type) IS	
		VARIABLE A: hash_type;	
		VARIABLE bufer: message_type; 		
	BEGIN	
		FOR i IN 0 TO 63 LOOP
			bufer(i)(7 DOWNTO 0):=X"00";
		END LOOP;	
		e8_degrouping(Q,A);
		H:=A; 
		FOR i IN 63 DOWNTO 0 LOOP 
			FOR j IN 7 DOWNTO 0 LOOP	
				H(i)(j):= A(i)(j); 
				H(i+64)(j):= A(i+64)(j) XOR bufer(i)(j);
			END LOOP;
		END LOOP;			
	END f8_reverse;	
		
	PROCEDURE hash_calculate (SIGNAL message_block: IN STD_LOGIC_VECTOR(511 DOWNTO 0); VARIABLE H: INOUT hash_type; VARIABLE Q: INOUT round_type) IS 		
		VARIABLE  bufer: message_type;
	BEGIN	
				bufer(0)(7 DOWNTO 0):=message_block(7 DOWNTO 0);
				bufer(1)(7 DOWNTO 0):=message_block(15 DOWNTO 8);
				bufer(2)(7 DOWNTO 0):=message_block(23 DOWNTO 16);
				bufer(3)(7 DOWNTO 0):=message_block(31 DOWNTO 24);
				bufer(4)(7 DOWNTO 0):=message_block(39 DOWNTO 32);
				bufer(5)(7 DOWNTO 0):=message_block(47 DOWNTO 40);
				bufer(6)(7 DOWNTO 0):=message_block(55 DOWNTO 48);
				bufer(7)(7 DOWNTO 0):=message_block(63 DOWNTO 56);
				bufer(8)(7 DOWNTO 0):=message_block(71 DOWNTO 64);
				bufer(9)(7 DOWNTO 0):=message_block(79 DOWNTO 72);
				bufer(10)(7 DOWNTO 0):=message_block(87 DOWNTO 80);
				bufer(11)(7 DOWNTO 0):=message_block(95 DOWNTO 88);
				bufer(12)(7 DOWNTO 0):=message_block(103 DOWNTO 96);
				bufer(13)(7 DOWNTO 0):=message_block(111 DOWNTO 104);
				bufer(14)(7 DOWNTO 0):=message_block(119 DOWNTO 112);
				bufer(15)(7 DOWNTO 0):=message_block(127 DOWNTO 120);
				bufer(16)(7 DOWNTO 0):=message_block(135 DOWNTO 128);
				bufer(17)(7 DOWNTO 0):=message_block(143 DOWNTO 136);
				bufer(18)(7 DOWNTO 0):=message_block(151 DOWNTO 144);
				bufer(19)(7 DOWNTO 0):=message_block(159 DOWNTO 152);
				bufer(20)(7 DOWNTO 0):=message_block(167 DOWNTO 160);
				bufer(21)(7 DOWNTO 0):=message_block(175 DOWNTO 168);
				bufer(22)(7 DOWNTO 0):=message_block(183 DOWNTO 176);
				bufer(23)(7 DOWNTO 0):=message_block(191 DOWNTO 184);
				bufer(24)(7 DOWNTO 0):=message_block(199 DOWNTO 192);
				bufer(25)(7 DOWNTO 0):=message_block(207 DOWNTO 200);
				bufer(26)(7 DOWNTO 0):=message_block(215 DOWNTO 208);
				bufer(27)(7 DOWNTO 0):=message_block(223 DOWNTO 216);
				bufer(28)(7 DOWNTO 0):=message_block(231 DOWNTO 224);
				bufer(29)(7 DOWNTO 0):=message_block(239 DOWNTO 232);
				bufer(30)(7 DOWNTO 0):=message_block(247 DOWNTO 240);
				bufer(31)(7 DOWNTO 0):=message_block(255 DOWNTO 248);
				bufer(32)(7 DOWNTO 0):=message_block(263 DOWNTO 256);
				bufer(33)(7 DOWNTO 0):=message_block(271 DOWNTO 264);
				bufer(34)(7 DOWNTO 0):=message_block(279 DOWNTO 272);
				bufer(35)(7 DOWNTO 0):=message_block(287 DOWNTO 280);
				bufer(36)(7 DOWNTO 0):=message_block(295 DOWNTO 288);
				bufer(37)(7 DOWNTO 0):=message_block(303 DOWNTO 296);
				bufer(38)(7 DOWNTO 0):=message_block(311 DOWNTO 304);
				bufer(39)(7 DOWNTO 0):=message_block(319 DOWNTO 312);
				bufer(40)(7 DOWNTO 0):=message_block(327 DOWNTO 320);
				bufer(41)(7 DOWNTO 0):=message_block(335 DOWNTO 328);
				bufer(42)(7 DOWNTO 0):=message_block(343 DOWNTO 336);
				bufer(43)(7 DOWNTO 0):=message_block(351 DOWNTO 344);
				bufer(44)(7 DOWNTO 0):=message_block(359 DOWNTO 352);
				bufer(45)(7 DOWNTO 0):=message_block(367 DOWNTO 360);
				bufer(46)(7 DOWNTO 0):=message_block(375 DOWNTO 368);
				bufer(47)(7 DOWNTO 0):=message_block(383 DOWNTO 376);
				bufer(48)(7 DOWNTO 0):=message_block(391 DOWNTO 384);
				bufer(49)(7 DOWNTO 0):=message_block(399 DOWNTO 392);
				bufer(50)(7 DOWNTO 0):=message_block(407 DOWNTO 400);
				bufer(51)(7 DOWNTO 0):=message_block(415 DOWNTO 408);
				bufer(52)(7 DOWNTO 0):=message_block(423 DOWNTO 416);
				bufer(53)(7 DOWNTO 0):=message_block(431 DOWNTO 424);
				bufer(54)(7 DOWNTO 0):=message_block(439 DOWNTO 432);
				bufer(55)(7 DOWNTO 0):=message_block(447 DOWNTO 440);
				bufer(56)(7 DOWNTO 0):=message_block(455 DOWNTO 448);
				bufer(57)(7 DOWNTO 0):=message_block(463 DOWNTO 456);
				bufer(58)(7 DOWNTO 0):=message_block(471 DOWNTO 464);
				bufer(59)(7 DOWNTO 0):=message_block(479 DOWNTO 472);
				bufer(60)(7 DOWNTO 0):=message_block(487 DOWNTO 480);
				bufer(61)(7 DOWNTO 0):=message_block(495 DOWNTO 488);
				bufer(62)(7 DOWNTO 0):=message_block(503 DOWNTO 496);
				bufer(63)(7 DOWNTO 0):=message_block(511 DOWNTO 504);		
		f8(bufer, H, Q);
	END hash_calculate;
	
	PROCEDURE hash_calculate_reverse (SIGNAL message_block: IN STD_LOGIC_VECTOR(511 DOWNTO 0); VARIABLE Q: INOUT round_type;VARIABLE H: INOUT hash_type) IS 		
		VARIABLE  bufer: message_type;
		VARIABLE A: hash_type;	
	BEGIN	
				A:=H;
				e8_degrouping(Q,A);
				bufer(0)(7 DOWNTO 0):=message_block(7 DOWNTO 0);
				bufer(1)(7 DOWNTO 0):=message_block(15 DOWNTO 8);
				bufer(2)(7 DOWNTO 0):=message_block(23 DOWNTO 16);
				bufer(3)(7 DOWNTO 0):=message_block(31 DOWNTO 24);
				bufer(4)(7 DOWNTO 0):=message_block(39 DOWNTO 32);
				bufer(5)(7 DOWNTO 0):=message_block(47 DOWNTO 40);
				bufer(6)(7 DOWNTO 0):=message_block(55 DOWNTO 48);
				bufer(7)(7 DOWNTO 0):=message_block(63 DOWNTO 56);
				bufer(8)(7 DOWNTO 0):=message_block(71 DOWNTO 64);
				bufer(9)(7 DOWNTO 0):=message_block(79 DOWNTO 72);
				bufer(10)(7 DOWNTO 0):=message_block(87 DOWNTO 80);
				bufer(11)(7 DOWNTO 0):=message_block(95 DOWNTO 88);
				bufer(12)(7 DOWNTO 0):=message_block(103 DOWNTO 96);
				bufer(13)(7 DOWNTO 0):=message_block(111 DOWNTO 104);
				bufer(14)(7 DOWNTO 0):=message_block(119 DOWNTO 112);
				bufer(15)(7 DOWNTO 0):=message_block(127 DOWNTO 120);
				bufer(16)(7 DOWNTO 0):=message_block(135 DOWNTO 128);
				bufer(17)(7 DOWNTO 0):=message_block(143 DOWNTO 136);
				bufer(18)(7 DOWNTO 0):=message_block(151 DOWNTO 144);
				bufer(19)(7 DOWNTO 0):=message_block(159 DOWNTO 152);
				bufer(20)(7 DOWNTO 0):=message_block(167 DOWNTO 160);
				bufer(21)(7 DOWNTO 0):=message_block(175 DOWNTO 168);
				bufer(22)(7 DOWNTO 0):=message_block(183 DOWNTO 176);
				bufer(23)(7 DOWNTO 0):=message_block(191 DOWNTO 184);
				bufer(24)(7 DOWNTO 0):=message_block(199 DOWNTO 192);
				bufer(25)(7 DOWNTO 0):=message_block(207 DOWNTO 200);
				bufer(26)(7 DOWNTO 0):=message_block(215 DOWNTO 208);
				bufer(27)(7 DOWNTO 0):=message_block(223 DOWNTO 216);
				bufer(28)(7 DOWNTO 0):=message_block(231 DOWNTO 224);
				bufer(29)(7 DOWNTO 0):=message_block(239 DOWNTO 232);
				bufer(30)(7 DOWNTO 0):=message_block(247 DOWNTO 240);
				bufer(31)(7 DOWNTO 0):=message_block(255 DOWNTO 248);
				bufer(32)(7 DOWNTO 0):=message_block(263 DOWNTO 256);
				bufer(33)(7 DOWNTO 0):=message_block(271 DOWNTO 264);
				bufer(34)(7 DOWNTO 0):=message_block(279 DOWNTO 272);
				bufer(35)(7 DOWNTO 0):=message_block(287 DOWNTO 280);
				bufer(36)(7 DOWNTO 0):=message_block(295 DOWNTO 288);
				bufer(37)(7 DOWNTO 0):=message_block(303 DOWNTO 296);
				bufer(38)(7 DOWNTO 0):=message_block(311 DOWNTO 304);
				bufer(39)(7 DOWNTO 0):=message_block(319 DOWNTO 312);
				bufer(40)(7 DOWNTO 0):=message_block(327 DOWNTO 320);
				bufer(41)(7 DOWNTO 0):=message_block(335 DOWNTO 328);
				bufer(42)(7 DOWNTO 0):=message_block(343 DOWNTO 336);
				bufer(43)(7 DOWNTO 0):=message_block(351 DOWNTO 344);
				bufer(44)(7 DOWNTO 0):=message_block(359 DOWNTO 352);
				bufer(45)(7 DOWNTO 0):=message_block(367 DOWNTO 360);
				bufer(46)(7 DOWNTO 0):=message_block(375 DOWNTO 368);
				bufer(47)(7 DOWNTO 0):=message_block(383 DOWNTO 376);
				bufer(48)(7 DOWNTO 0):=message_block(391 DOWNTO 384);
				bufer(49)(7 DOWNTO 0):=message_block(399 DOWNTO 392);
				bufer(50)(7 DOWNTO 0):=message_block(407 DOWNTO 400);
				bufer(51)(7 DOWNTO 0):=message_block(415 DOWNTO 408);
				bufer(52)(7 DOWNTO 0):=message_block(423 DOWNTO 416);
				bufer(53)(7 DOWNTO 0):=message_block(431 DOWNTO 424);
				bufer(54)(7 DOWNTO 0):=message_block(439 DOWNTO 432);
				bufer(55)(7 DOWNTO 0):=message_block(447 DOWNTO 440);
				bufer(56)(7 DOWNTO 0):=message_block(455 DOWNTO 448);
				bufer(57)(7 DOWNTO 0):=message_block(463 DOWNTO 456);
				bufer(58)(7 DOWNTO 0):=message_block(471 DOWNTO 464);
				bufer(59)(7 DOWNTO 0):=message_block(479 DOWNTO 472);
				bufer(60)(7 DOWNTO 0):=message_block(487 DOWNTO 480);
				bufer(61)(7 DOWNTO 0):=message_block(495 DOWNTO 488);
				bufer(62)(7 DOWNTO 0):=message_block(503 DOWNTO 496);
				bufer(63)(7 DOWNTO 0):=message_block(511 DOWNTO 504);		
		FOR i IN 63 DOWNTO 0 LOOP 
			FOR j IN 7 DOWNTO 0 LOOP	
				H(i)(j):= A(i)(j); 
				H(i+64)(j):= A(i+64)(j) XOR bufer(i)(j);
			END LOOP;
		END LOOP;			
	END hash_calculate_reverse;	
		
	PROCEDURE hash_final_func (VARIABLE H: INOUT hash_type; VARIABLE hash_final: OUT STD_LOGIC_VECTOR (383 DOWNTO 0)) IS 		
	BEGIN					
			hash_final(383 DOWNTO 376):= H(80)(7 DOWNTO 0);
			hash_final(375 DOWNTO 368):= H(81)(7 DOWNTO 0);
			hash_final(367 DOWNTO 360):= H(82)(7 DOWNTO 0);
			hash_final(359 DOWNTO 352):= H(83)(7 DOWNTO 0);
			hash_final(351 DOWNTO 344):= H(84)(7 DOWNTO 0);
			hash_final(343 DOWNTO 336):= H(85)(7 DOWNTO 0);
			hash_final(335 DOWNTO 328):= H(86)(7 DOWNTO 0);
			hash_final(327 DOWNTO 320):= H(87)(7 DOWNTO 0);
			hash_final(319 DOWNTO 312):= H(88)(7 DOWNTO 0);
			hash_final(311 DOWNTO 304):= H(89)(7 DOWNTO 0);
			hash_final(303 DOWNTO 296):= H(90)(7 DOWNTO 0);
			hash_final(295 DOWNTO 288):= H(91)(7 DOWNTO 0);
			hash_final(287 DOWNTO 280):= H(92)(7 DOWNTO 0);
			hash_final(279 DOWNTO 272):= H(93)(7 DOWNTO 0);
			hash_final(271 DOWNTO 264):= H(94)(7 DOWNTO 0);
			hash_final(263 DOWNTO 256):= H(95)(7 DOWNTO 0);
			hash_final(255 DOWNTO 248):= H(96)(7 DOWNTO 0);
			hash_final(247 DOWNTO 240):= H(97)(7 DOWNTO 0);
			hash_final(239 DOWNTO 232):= H(98)(7 DOWNTO 0);
			hash_final(231 DOWNTO 224):= H(99)(7 DOWNTO 0);
			hash_final(223 DOWNTO 216):= H(100)(7 DOWNTO 0);
			hash_final(215 DOWNTO 208):= H(101)(7 DOWNTO 0);
			hash_final(207 DOWNTO 200):= H(102)(7 DOWNTO 0);
			hash_final(199 DOWNTO 192):= H(103)(7 DOWNTO 0);
			hash_final(191 DOWNTO 184):= H(104)(7 DOWNTO 0);
			hash_final(183 DOWNTO 176):= H(105)(7 DOWNTO 0);
			hash_final(175 DOWNTO 168):= H(106)(7 DOWNTO 0);
			hash_final(167 DOWNTO 160):= H(107)(7 DOWNTO 0);
			hash_final(159 DOWNTO 152):= H(108)(7 DOWNTO 0);
			hash_final(151 DOWNTO 144):= H(109)(7 DOWNTO 0);
			hash_final(143 DOWNTO 136):= H(110)(7 DOWNTO 0);
			hash_final(135 DOWNTO 128):= H(111)(7 DOWNTO 0);
			hash_final(127 DOWNTO 120):= H(112)(7 DOWNTO 0);
			hash_final(119 DOWNTO 112):= H(113)(7 DOWNTO 0);
			hash_final(111 DOWNTO 104):= H(114)(7 DOWNTO 0);
			hash_final(103 DOWNTO 96):= H(115)(7 DOWNTO 0);
			hash_final(95 DOWNTO 88):= H(116)(7 DOWNTO 0);
			hash_final(87 DOWNTO 80):= H(117)(7 DOWNTO 0);
			hash_final(79 DOWNTO 72):= H(118)(7 DOWNTO 0);
			hash_final(71 DOWNTO 64):= H(119)(7 DOWNTO 0);
			hash_final(63 DOWNTO 56):= H(120)(7 DOWNTO 0);
			hash_final(55 DOWNTO 48):= H(121)(7 DOWNTO 0);
			hash_final(47 DOWNTO 40):= H(122)(7 DOWNTO 0);
			hash_final(39 DOWNTO 32):= H(123)(7 DOWNTO 0);
			hash_final(31 DOWNTO 24):= H(124)(7 DOWNTO 0);
			hash_final(23 DOWNTO 16):= H(125)(7 DOWNTO 0);
			hash_final(15 DOWNTO 8):= H(126)(7 DOWNTO 0);
			hash_final(7 DOWNTO 0):= H(127)(7 DOWNTO 0);
	END hash_final_func;
	
	PROCEDURE hash_initial (VARIABLE H: INOUT hash_type;VARIABLE Q: OUT round_type) IS 		
		VARIABLE hashbitlen_var: STD_LOGIC_VECTOR (15 DOWNTO 0);
		VARIABLE bufer: message_type;
	BEGIN	
		hashbitlen_var:=hashbitlen;
		FOR i IN 63 DOWNTO 0 LOOP
			bufer(i)(7 DOWNTO 0):= X"00";
		END LOOP;
		FOR i IN 127 DOWNTO 0 LOOP			
			H(i)(7 DOWNTO 0):= X"00";
		END LOOP;	
		H(1) := hashbitlen_var(7 DOWNTO 0);
		H(0) := hashbitlen_var(15 DOWNTO 8);
		f8(bufer, H,Q);   
	END hash_initial;
	
END JH_384_paket;